@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,7 @@
 #include "wayland-external-exports.h"
 #include "wayland-egldisplay.h"
 #include "wayland-eglstream.h"
-#include "wayland-eglsurface.h"
+#include "wayland-eglsurface-internal.h"
 #include "wayland-eglswap.h"
 #include "wayland-eglutils.h"
 #include "wayland-eglhandle.h"
@@ -50,6 +50,7 @@ static const WlEglHook wlEglHooksMap[] = {
     { "eglInitialize",                     wlEglInitializeHook },
     { "eglQueryDisplayAttribEXT",          wlEglQueryDisplayAttribHook },
     { "eglQueryDisplayAttribKHR",          wlEglQueryDisplayAttribHook },
+    { "eglQuerySurface",                   wlEglQuerySurfaceHook },
     { "eglQueryWaylandBufferWL",           wlEglQueryNativeResourceHook },
     { "eglSwapBuffers",                    wlEglSwapBuffersHook },
     { "eglSwapBuffersWithDamageKHR",       wlEglSwapBuffersWithDamageHook },
